#!/usr/bin/python3

# MIT License

# Copyright (c) 2024 Thomas Williams

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import config
import psutil
import os
import time
import log
import requests
import threading
from functools import partial
from concurrent.futures import ThreadPoolExecutor, as_completed
from bs4 import BeautifulSoup

def loadUrl(url):
    
    headers = { 'User-Agent': 'Monutil monitor' }

    response = requests.get(url, timeout=config.urlTimeout, headers=headers)
    return response

def prepareUrl(src, baseUrl):

    if not src.startswith("http://") and not src.startswith("https://"):
        return baseUrl.rstrip("/") + "/" + src.lstrip("/")
    return src

def monitorHost():

    while True:

        load1, load5, load15 = psutil.getloadavg() # this takes time to warm up if not running script on *nix 
        loadavg = round((load1/os.cpu_count()) * 100, 2)

        total_memory, used_memory, free_memory = map(
                int, os.popen('free -t -m').readlines()[-1].split()[1:])
        memory = round((used_memory/total_memory) * 100, 2)

        print("CPU %: " + str(loadavg))
        print("Memory %: " + str(memory))

        time.sleep(config.hostMonitoringPeriod)

def monitorUrls():

    while True:
        
        for url in config.urls:
            
            baseUrl = url
            urlFail = False

            startTime = time.time()
            request = loadUrl(url)

            if request.status_code == 200:

                html = BeautifulSoup(request.content, 'html.parser')
                imageUrls = [img['src'] for img in html.find_all('img')]

                with ThreadPoolExecutor(max_workers=config.maxWorkers) as executor:
            
                    responses = [executor.submit(loadUrl, prepareUrl(url, baseUrl)) for url in imageUrls]    
                    responses = [future.result() for future in as_completed(responses)]

                for response in responses:
            
                    if not response.status_code == 200:
                        urlFail = True

                endTime = time.time()
                timeDiff = endTime - startTime
            
                print(timeDiff)
        
            else:
                urlFail = True

        time.sleep(config.urlMonitoringPeriod)

def main():
    
    hostMonitorThread = threading.Thread(target=monitorHost)
    urlMonitorThread = threading.Thread(target=monitorUrls)
    
    hostMonitorThread.start() 
    urlMonitorThread.start()

    hostMonitorThread.join()
    urlMonitorThread.join()

if __name__ == "__main__":
    main()
